/*
  Copyright 2010-2012 Grégory Soutadé

  This file is part of KissCount.

  KissCount is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  KissCount is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with KissCount.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef IMPORTPANEL_H
#define IMPORTPANEL_H

#include <QtGui>
#include <QCheckBox>

#include "view.hpp"
#include <model/model.hpp>
#include <model/import/ImportEngine.hpp>

#include "grid/GridAccount.hpp"

class ImportPanel: public KissPanel
{
    Q_OBJECT;

public:
    ImportPanel(KissCount* kiss, wxUI *parent, bool lowResolution);

    KissPanel* CreatePanel();
    QString GetToolTip();
    void OnShow();

private slots:
    void OnFile();
    void OnFileEnter();
    void OnLoadOperations();
    void OnIntegrate();
    void OnOperationModified(int row, int col);

private:
    QHBoxLayout *_hbox;
    QTableWidget* _accountsGrid, *_categoriesGrid, *_tagsGrid;
    QLineEdit* _fileTxt;
    GridAccount* _operationsGrid;
    ImportEngine* _importEngine;
    QPushButton* _buttonLoadOperations, *_buttonIntegrate;
    QCheckBox *_checkSaveImportPatterns;
    std::vector<Operation>* _operations;

    std::vector<Account> _unresolvedAccounts;
    std::vector<Category> _unresolvedCategories;
    std::vector<Tag> _unresolvedTags;
    std::map<int, int> _resolvedAccounts;
    std::map<int, int> _resolvedCategories;
    std::map<int, int> _resolvedTags;

    void ProcessFile();
};

#endif
