/*
  Copyright 2010-2012 Grégory Soutadé

  This file is part of KissCount.

  KissCount is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  KissCount is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with KissCount.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SEARCHPANEL_H
#define SEARCHPANEL_H

#include <QtGui>

#include "view.hpp"
#include "grid/GridAccount.hpp"
#include "AccountPanel.hpp"
#include "SearchBanner.hpp"
#include "CostRepartitionBanner.hpp"

#include <model/model.hpp>

class GridAccount;

class SearchPanel: public KissPanel
{
    Q_OBJECT;

public:
    SearchPanel(KissCount* kiss, wxUI *parent, bool lowResolution);
    ~SearchPanel();

    KissPanel* CreatePanel();
    QString GetToolTip();
    void OnShow();

private slots:
    void OnButtonSearch();
    void OnOperationModified();

    void OnButtonChangeAccount();
    void OnButtonChangeCategory();
    void OnButtonChangeTag();
    void OnButtonRename();

private:
    void UpdateCostRepartition();

    std::vector<Operation> *_operations;
    SearchBanner* _banner;
    GridAccount *_grid;
    CostRepartitionBanner *_costRepartitionBanner;
    QString* _categories;  
    int *_categoriesValues;
    std::map<int, int> _categoriesIndexes;
    QPushButton* _searchButton, *_renameButton, *_changeAccountButton, *_changeCategoryButton, *_changeTagButton;

    static void OnEnter(void* caller);
};

#endif
